@@ -8,9 +8,12 @@ Provides session-based PDF manipulation operations with strict validation.
 import json
 import os
 from pathlib import Path
-from typing import List, Optional, Union, BinaryIO
+from typing import List, Optional, Union, BinaryIO, Mapping, Any
 
 import requests
+from dotenv import load_dotenv
+
+load_dotenv()
 
 from . import ParagraphBuilder
 from .exceptions import (
@@ -22,21 +25,32 @@ from .exceptions import (
 )
 from .image_builder import ImageBuilder
 from .models import (
-    ObjectRef, Position, ObjectType, Font, Image, Paragraph, FormFieldRef,
-    FindRequest, DeleteRequest, MoveRequest, AddRequest, ModifyRequest, ModifyTextRequest, ChangeFormFieldRequest,
-    ShapeType, PositionMode
+    ObjectRef, Position, ObjectType, Font, Image, Paragraph, FormFieldRef, TextObjectRef, PageRef,
+    FindRequest, DeleteRequest, MoveRequest, PageMoveRequest, AddRequest, ModifyRequest, ModifyTextRequest,
+    ChangeFormFieldRequest, CommandResult,
+    ShapeType, PositionMode, PageSize, Orientation
 )
 from .paragraph_builder import ParagraphPageBuilder
 from .types import PathObject, ParagraphObject, TextLineObject, ImageObject, FormObject, FormFieldObject
 
 
 class PageClient:
-    def __init__(self, page_index: int, root: "PDFDancer"):
+    def __init__(self, page_index: int, root: "PDFDancer", page_size: Optional[PageSize] = None,
+                 orientation: Optional[Union[Orientation, str]] = Orientation.PORTRAIT):
         self.page_index = page_index
         self.root = root
         self.object_type = ObjectType.PAGE
         self.position = Position.at_page(page_index)
         self.internal_id = f"PAGE-{page_index}"
+        self.page_size = page_size
+        if isinstance(orientation, str):
+            normalized = orientation.strip().upper()
+            try:
+                self.orientation = Orientation(normalized)
+            except ValueError:
+                self.orientation = normalized
+        else:
+            self.orientation = orientation
 
     def select_paths_at(self, x: float, y: float) -> List[PathObject]:
         # noinspection PyProtectedMember
@@ -52,6 +66,18 @@ class PageClient:
         # noinspection PyProtectedMember
         return self.root._to_paragraph_objects(self.root._find_paragraphs(position))
 
+    def select_paragraphs_matching(self, pattern):
+        position = Position.at_page(self.page_index)
+        position.text_pattern = pattern
+        # noinspection PyProtectedMember
+        return self.root._to_paragraph_objects(self.root._find_paragraphs(position))
+
+    def select_text_lines_matching(self, pattern: str) -> List[TextLineObject]:
+        position = Position.at_page(self.page_index)
+        position.text_pattern = pattern
+        # noinspection PyProtectedMember
+        return self.root._to_textline_objects(self.root._find_text_lines(position))
+
     def select_paragraphs_at(self, x: float, y: float) -> List[ParagraphObject]:
         position = Position.at_page_coordinates(self.page_index, x, y)
         # noinspection PyProtectedMember
@@ -68,6 +94,11 @@ class PageClient:
         # noinspection PyProtectedMember
         return self.root._to_textline_objects(self.root._find_text_lines(position))
 
+    def select_text_lines_at(self, x, y) -> List[TextLineObject]:
+        position = Position.at_page_coordinates(self.page_index, x, y)
+        # noinspection PyProtectedMember
+        return self.root._to_textline_objects(self.root._find_text_lines(position))
+
     def select_images(self) -> List[ImageObject]:
         # noinspection PyProtectedMember
         return self.root._to_image_objects(self.root._find_images(Position.at_page(self.page_index)))
@@ -104,20 +135,71 @@ class PageClient:
         return self.root._to_form_field_objects(self.root._find_form_fields(position))
 
     @classmethod
-    def from_ref(cls, root: 'PDFDancer', object_ref: ObjectRef) -> 'PageClient':
-        page_client = PageClient(page_index=object_ref.position.page_index, root=root)
+    def from_ref(cls, root: 'PDFDancer', page_ref: PageRef) -> 'PageClient':
+        page_client = PageClient(
+            page_index=page_ref.position.page_index,
+            root=root,
+            page_size=page_ref.page_size,
+            orientation=page_ref.orientation
+        )
+        page_client.internal_id = page_ref.internal_id
+        if page_ref.position is not None:
+            page_client.position = page_ref.position
+            page_client.page_index = page_ref.position.page_index
         return page_client
 
     def delete(self) -> bool:
         # noinspection PyProtectedMember
         return self.root._delete_page(self._ref())
 
+    def move_to(self, target_page_index: int) -> bool:
+        """Move this page to a different index within the document."""
+        if target_page_index is None or target_page_index < 0:
+            raise ValidationException(f"Target page index must be >= 0, got {target_page_index}")
+
+        # noinspection PyProtectedMember
+        moved = self.root._move_page(self.page_index, target_page_index)
+        if moved:
+            self.page_index = target_page_index
+            self.position = Position.at_page(target_page_index)
+        return moved
+
     def _ref(self):
         return ObjectRef(internal_id=self.internal_id, position=self.position, type=self.object_type)
 
     def new_paragraph(self):
         return ParagraphPageBuilder(self.root, self.page_index)
 
+    def select_paths(self):
+        # noinspection PyProtectedMember
+        return self.root._to_path_objects(self.root._find_paths(Position.at_page(self.page_index)))
+
+    def select_elements(self):
+        """
+        Select all elements (paragraphs, images, paths, forms) on this page.
+
+        Returns:
+            List of all PDF objects on this page
+        """
+        result = []
+        result.extend(self.select_paragraphs())
+        result.extend(self.select_text_lines())
+        result.extend(self.select_images())
+        result.extend(self.select_paths())
+        result.extend(self.select_forms())
+        result.extend(self.select_form_fields())
+        return result
+
+    @property
+    def size(self):
+        """Property alias for page size."""
+        return self.page_size
+
+    @property
+    def page_orientation(self):
+        """Property alias for orientation."""
+        return self.orientation
+
 
 class PDFDancer:
     """
@@ -149,6 +231,21 @@ class PDFDancer:
         Returns:
             A ready-to-use `PDFDancer` client instance.
         """
+        resolved_token = cls._resolve_token(token)
+        resolved_base_url = cls._resolve_base_url(base_url)
+
+        return PDFDancer(resolved_token, pdf_data, resolved_base_url, timeout)
+
+    @classmethod
+    def _resolve_base_url(cls, base_url: Optional[str]) -> Optional[str]:
+        env_base_url = os.getenv("PDFDANCER_BASE_URL")
+        resolved_base_url = base_url or (env_base_url.strip() if env_base_url and env_base_url.strip() else None)
+        if resolved_base_url is None:
+            resolved_base_url = "https://api.pdfdancer.com"
+        return resolved_base_url
+
+    @classmethod
+    def _resolve_token(cls, token: Optional[str]) -> Optional[str]:
         resolved_token = token.strip() if token and token.strip() else None
         if resolved_token is None:
             env_token = os.getenv("PDFDANCER_TOKEN")
@@ -159,13 +256,63 @@ class PDFDancer:
                 "Missing PDFDancer API token. Pass a token via the `token` argument "
                 "or set the PDFDANCER_TOKEN environment variable."
             )
+        return resolved_token
 
-        env_base_url = os.getenv("PDFDANCER_BASE_URL")
-        resolved_base_url = base_url or (env_base_url.strip() if env_base_url and env_base_url.strip() else None)
-        if resolved_base_url is None:
-            resolved_base_url = "https://api.pdfdancer.com"
+    @classmethod
+    def new(cls,
+            token: Optional[str] = None,
+            base_url: Optional[str] = None,
+            timeout: float = 30.0,
+            page_size: Optional[Union[PageSize, str, Mapping[str, Any]]] = None,
+            orientation: Optional[Union[Orientation, str]] = None,
+            initial_page_count: int = 1) -> "PDFDancer":
+        """
+        Create a new blank PDF document with optional configuration.
 
-        return PDFDancer(resolved_token, pdf_data, resolved_base_url, timeout)
+        Args:
+            token: Override for the API token; falls back to `PDFDANCER_TOKEN` environment variable.
+            base_url: Override for the API base URL; falls back to `PDFDANCER_BASE_URL`
+                or defaults to `https://api.pdfdancer.com`.
+            timeout: HTTP read timeout in seconds.
+            page_size: Page size for the PDF (default: A4). Accepts `PageSize`, a standard name string, or a
+                mapping with `width`/`height` values.
+            orientation: Page orientation (default: PORTRAIT). Can be Orientation enum or string.
+            initial_page_count: Number of initial blank pages (default: 1).
+
+        Returns:
+            A ready-to-use `PDFDancer` client instance with a blank PDF.
+        """
+        resolved_token = cls._resolve_token(token)
+        resolved_base_url = cls._resolve_base_url(base_url)
+
+        # Create a new instance that will call _create_blank_pdf_session
+        instance = object.__new__(cls)
+
+        # Initialize instance variables
+        if not resolved_token or not resolved_token.strip():
+            raise ValidationException("Authentication token cannot be null or empty")
+
+        instance._token = resolved_token.strip()
+        instance._base_url = resolved_base_url.rstrip('/')
+        instance._read_timeout = timeout
+
+        # Create HTTP session for connection reuse
+        instance._session = requests.Session()
+        instance._session.headers.update({
+            'Authorization': f'Bearer {instance._token}'
+        })
+
+        # Create blank PDF session
+        instance._session_id = instance._create_blank_pdf_session(
+            page_size=page_size,
+            orientation=orientation,
+            initial_page_count=initial_page_count
+        )
+
+        # Set pdf_bytes to None since we don't have the PDF bytes yet
+        instance._pdf_bytes = None
+
+        return instance
 
     def __init__(self, token: str, pdf_data: Union[bytes, Path, str, BinaryIO],
                  base_url: str, read_timeout: float = 0):
@@ -205,7 +352,8 @@ class PDFDancer:
         # Create session - equivalent to Java constructor behavior
         self._session_id = self._create_session()
 
-    def _process_pdf_data(self, pdf_data: Union[bytes, Path, str, BinaryIO]) -> bytes:
+    @staticmethod
+    def _process_pdf_data(pdf_data: Union[bytes, Path, str, BinaryIO]) -> bytes:
         """
         Process PDF data from various input types with strict validation.
         """
@@ -297,6 +445,22 @@ class PDFDancer:
                 f"Server response: {details}"
             )
 
+    @staticmethod
+    def _cleanup_url_path(base_url: str, path: str) -> str:
+        """
+        Combine base_url and path, ensuring no double slashes.
+
+        Args:
+            base_url: Base URL (may or may not have trailing slash)
+            path: Path segment (may or may not have leading slash)
+
+        Returns:
+            Combined URL with no double slashes
+        """
+        base = base_url.rstrip('/')
+        path = path.lstrip('/')
+        return f"{base}/{path}"
+
     def _create_session(self) -> str:
         """
         Creates a new PDF processing session by uploading the PDF data.
@@ -307,7 +471,7 @@ class PDFDancer:
             }
 
             response = self._session.post(
-                f"{self._base_url}/session/create",
+                self._cleanup_url_path(self._base_url, "/session/create"),
                 files=files,
                 timeout=self._read_timeout if self._read_timeout > 0 else None
             )
@@ -327,6 +491,76 @@ class PDFDancer:
             raise HttpClientException(f"Failed to create session: {error_message}",
                                       response=getattr(e, 'response', None), cause=e) from None
 
+    def _create_blank_pdf_session(self,
+                                  page_size: Optional[Union[PageSize, str, Mapping[str, Any]]] = None,
+                                  orientation: Optional[Union[Orientation, str]] = None,
+                                  initial_page_count: int = 1) -> str:
+        """
+        Creates a new PDF processing session with a blank PDF document.
+
+        Args:
+            page_size: Page size (default: A4). Accepts `PageSize`, a standard name string, or a
+                mapping with `width`/`height` values.
+            orientation: Page orientation (default: PORTRAIT). Can be Orientation enum or string.
+            initial_page_count: Number of initial pages (default: 1)
+
+        Returns:
+            Session ID for the newly created blank PDF
+
+        Raises:
+            SessionException: If session creation fails
+            HttpClientException: If HTTP communication fails
+        """
+        try:
+            # Build request payload
+            request_data = {}
+
+            # Handle page_size - convert to type-safe object with dimensions
+            if page_size is not None:
+                try:
+                    request_data['pageSize'] = PageSize.coerce(page_size).to_dict()
+                except ValueError as exc:
+                    raise ValidationException(str(exc)) from exc
+                except TypeError:
+                    raise ValidationException(f"Invalid page_size type: {type(page_size)}")
+
+            # Handle orientation
+            if orientation is not None:
+                if isinstance(orientation, Orientation):
+                    request_data['orientation'] = orientation.value
+                elif isinstance(orientation, str):
+                    request_data['orientation'] = orientation
+                else:
+                    raise ValidationException(f"Invalid orientation type: {type(orientation)}")
+
+            # Handle initial_page_count with validation
+            if initial_page_count < 1:
+                raise ValidationException(f"Initial page count must be at least 1, got {initial_page_count}")
+            request_data['initialPageCount'] = initial_page_count
+
+            headers = {'Content-Type': 'application/json'}
+            response = self._session.post(
+                self._cleanup_url_path(self._base_url, "/session/new"),
+                json=request_data,
+                headers=headers,
+                timeout=self._read_timeout if self._read_timeout > 0 else None
+            )
+
+            self._handle_authentication_error(response)
+            response.raise_for_status()
+            session_id = response.text.strip()
+
+            if not session_id:
+                raise SessionException("Server returned empty session ID")
+
+            return session_id
+
+        except requests.exceptions.RequestException as e:
+            self._handle_authentication_error(getattr(e, 'response', None))
+            error_message = self._extract_error_message(getattr(e, 'response', None))
+            raise HttpClientException(f"Failed to create blank PDF session: {error_message}",
+                                      response=getattr(e, 'response', None), cause=e) from None
+
     def _make_request(self, method: str, path: str, data: Optional[dict] = None,
                       params: Optional[dict] = None) -> requests.Response:
         """
@@ -340,14 +574,14 @@ class PDFDancer:
         try:
             response = self._session.request(
                 method=method,
-                url=f"{self._base_url}{path}",
+                url=self._cleanup_url_path(self._base_url, path),
                 json=data,
                 params=params,
                 headers=headers,
                 timeout=self._read_timeout if self._read_timeout > 0 else None
             )
 
-            # Handle FontNotFoundException specifically like Java client
+            # Handle FontNotFoundException
             if response.status_code == 404:
                 try:
                     error_data = response.json()
@@ -366,8 +600,6 @@ class PDFDancer:
             raise HttpClientException(f"API request failed: {error_message}", response=getattr(e, 'response', None),
                                       cause=e) from None
 
-    # Search Operations - matching Java client exactly
-
     def _find(self, object_type: Optional[ObjectType] = None, position: Optional[Position] = None) -> List[ObjectRef]:
         """
         Searches for PDF objects matching the specified criteria.
@@ -388,17 +620,21 @@ class PDFDancer:
         objects_data = response.json()
         return [self._parse_object_ref(obj_data) for obj_data in objects_data]
 
-    def select_paragraphs(self) -> List[ParagraphObject]:
+    def select_paragraphs(self) -> List[TextObjectRef]:
         """
-        Searches for paragraph objects at the specified position.
+        Searches for paragraph objects returning TextObjectRef with hierarchical structure.
         """
-        return self._to_paragraph_objects(self._find(ObjectType.PARAGRAPH, None))
+        return self._find_paragraphs(None)
 
-    def _find_paragraphs(self, position: Optional[Position] = None) -> List[ObjectRef]:
+    def _find_paragraphs(self, position: Optional[Position] = None) -> List[TextObjectRef]:
         """
-        Searches for paragraph objects at the specified position.
+        Searches for paragraph objects returning TextObjectRef with hierarchical structure.
         """
-        return self._find(ObjectType.PARAGRAPH, position)
+        request_data = FindRequest(ObjectType.PARAGRAPH, position).to_dict()
+        response = self._make_request('POST', '/pdf/find', data=request_data)
+
+        objects_data = response.json()
+        return [self._parse_text_object_ref(obj_data) for obj_data in objects_data]
 
     def _find_images(self, position: Optional[Position] = None) -> List[ObjectRef]:
         """
@@ -471,35 +707,53 @@ class PDFDancer:
         """
         return self._find(ObjectType.PATH, position)
 
-    def _find_text_lines(self, position: Optional[Position] = None) -> List[ObjectRef]:
+    def _find_text_lines(self, position: Optional[Position] = None) -> List[TextObjectRef]:
         """
-        Searches for text line objects at the specified position.
+        Searches for text line objects returning TextObjectRef with hierarchical structure.
         """
-        return self._find(ObjectType.TEXT_LINE, position)
+        request_data = FindRequest(ObjectType.TEXT_LINE, position).to_dict()
+        response = self._make_request('POST', '/pdf/find', data=request_data)
+
+        objects_data = response.json()
+        return [self._parse_text_object_ref(obj_data) for obj_data in objects_data]
 
     def select_text_lines(self) -> List[TextLineObject]:
         """
-        Searches for text line objects at the specified position.
+        Searches for text line objects returning TextLineObject wrappers.
         """
-        return self._to_textline_objects(self._find(ObjectType.TEXT_LINE, None))
+        return self._to_textline_objects(self._find_text_lines(None))
 
     def page(self, page_index: int) -> PageClient:
-        return PageClient(page_index, self)
+        """
+        Get a specific page by index, fetching page properties from the server.
+
+        Args:
+            page_index: The 0-based page index
+
+        Returns:
+            PageClient with page properties populated
+        """
+        page_ref = self._get_page(page_index)
+        if page_ref:
+            return PageClient.from_ref(self, page_ref)
+        else:
+            # Fallback to basic PageClient if page not found
+            return PageClient(page_index, self)
 
     # Page Operations
 
     def pages(self) -> List[PageClient]:
         return self._to_page_objects(self._get_pages())
 
-    def _get_pages(self) -> List[ObjectRef]:
+    def _get_pages(self) -> List[PageRef]:
         """
         Retrieves references to all pages in the PDF document.
         """
         response = self._make_request('POST', '/pdf/page/find')
         pages_data = response.json()
-        return [self._parse_object_ref(page_data) for page_data in pages_data]
+        return [self._parse_page_ref(page_data) for page_data in pages_data]
 
-    def _get_page(self, page_index: int) -> Optional[ObjectRef]:
+    def _get_page(self, page_index: int) -> Optional[PageRef]:
         """
         Retrieves a reference to a specific page by its page index.
 
@@ -507,7 +761,7 @@ class PDFDancer:
             page_index: The page index to retrieve (1-based indexing)
 
         Returns:
-            Object reference for the specified page, or None if not found
+            Page reference for the specified page, or None if not found
         """
         if page_index < 0:
             raise ValidationException(f"Page index must be >= 0, got {page_index}")
@@ -519,7 +773,7 @@ class PDFDancer:
         if not pages_data:
             return None
 
-        return self._parse_object_ref(pages_data[0])
+        return self._parse_page_ref(pages_data[0])
 
     def _delete_page(self, page_ref: ObjectRef) -> bool:
         """
@@ -539,6 +793,25 @@ class PDFDancer:
         response = self._make_request('DELETE', '/pdf/page/delete', data=request_data)
         return response.json()
 
+    def move_page(self, from_page_index: int, to_page_index: int) -> bool:
+        """Move a page to a different index within the document."""
+        return self._move_page(from_page_index, to_page_index)
+
+    def _move_page(self, from_page_index: int, to_page_index: int) -> bool:
+        """Internal helper to perform the page move operation."""
+        for value, label in ((from_page_index, "from_page_index"), (to_page_index, "to_page_index")):
+            if value is None:
+                raise ValidationException(f"{label} cannot be null")
+            if not isinstance(value, int):
+                raise ValidationException(f"{label} must be an integer, got {type(value)}")
+            if value < 0:
+                raise ValidationException(f"{label} must be >= 0, got {value}")
+
+        request_data = PageMoveRequest(from_page_index, to_page_index).to_dict()
+        response = self._make_request('PUT', '/pdf/page/move', data=request_data)
+        result = response.json()
+        return bool(result)
+
     # Manipulation Operations
 
     def _delete(self, object_ref: ObjectRef) -> bool:
@@ -634,11 +907,15 @@ class PDFDancer:
     def new_paragraph(self) -> ParagraphBuilder:
         return ParagraphBuilder(self)
 
+    def new_page(self):
+        response = self._make_request('POST', '/pdf/page/add', data=None)
+        return self._parse_page_ref(response.json())
+
     def new_image(self) -> ImageBuilder:
         return ImageBuilder(self)
 
     # Modify Operations
-    def _modify_paragraph(self, object_ref: ObjectRef, new_paragraph: Union[Paragraph, str]) -> bool:
+    def _modify_paragraph(self, object_ref: ObjectRef, new_paragraph: Union[Paragraph, str]) -> CommandResult:
         """
         Modifies a paragraph object or its text content.
 
@@ -652,20 +929,20 @@ class PDFDancer:
         if object_ref is None:
             raise ValidationException("Object reference cannot be null")
         if new_paragraph is None:
-            raise ValidationException("New paragraph cannot be null")
+            return CommandResult.empty("ModifyParagraph", object_ref.internal_id)
 
         if isinstance(new_paragraph, str):
-            # Text modification
+            # Text modification - returns CommandResult
             request_data = ModifyTextRequest(object_ref, new_paragraph).to_dict()
             response = self._make_request('PUT', '/pdf/text/paragraph', data=request_data)
+            return CommandResult.from_dict(response.json())
         else:
             # Object modification
             request_data = ModifyRequest(object_ref, new_paragraph).to_dict()
             response = self._make_request('PUT', '/pdf/modify', data=request_data)
+            return CommandResult.from_dict(response.json())
 
-        return response.json()
-
-    def _modify_text_line(self, object_ref: ObjectRef, new_text: str) -> bool:
+    def _modify_text_line(self, object_ref: ObjectRef, new_text: str) -> CommandResult:
         """
         Modifies a text line object.
 
@@ -683,7 +960,7 @@ class PDFDancer:
 
         request_data = ModifyTextRequest(object_ref, new_text).to_dict()
         response = self._make_request('PUT', '/pdf/text/line', data=request_data)
-        return response.json()
+        return CommandResult.from_dict(response.json())
 
     # Font Operations
 
@@ -765,7 +1042,7 @@ class PDFDancer:
 
             headers = {'X-Session-Id': self._session_id}
             response = self._session.post(
-                f"{self._base_url}/font/register",
+                self._cleanup_url_path(self._base_url, "/font/register"),
                 files=files,
                 headers=headers,
                 timeout=30
@@ -835,7 +1112,7 @@ class PDFDancer:
             type=object_type
         )
 
-    def _parse_form_field_ref(self, obj_data: dict) -> ObjectRef:
+    def _parse_form_field_ref(self, obj_data: dict) -> FormFieldRef:
         """Parse JSON object data into ObjectRef instance."""
         position_data = obj_data.get('position', {})
         position = self._parse_position(position_data) if position_data else None
@@ -874,6 +1151,105 @@ class PDFDancer:
 
         return position
 
+    def _parse_text_object_ref(self, obj_data: dict, fallback_id: Optional[str] = None) -> TextObjectRef:
+        """Parse JSON object data into TextObjectRef instance with hierarchical structure."""
+        position_data = obj_data.get('position', {})
+        position = self._parse_position(position_data) if position_data else Position()
+
+        object_type = ObjectType(obj_data.get('type', 'TEXT_LINE'))
+        line_spacings = obj_data.get('lineSpacings') if isinstance(obj_data.get('lineSpacings'), list) else None
+        internal_id = obj_data.get('internalId', fallback_id or '')
+
+        color = None
+        color_data = obj_data.get('color')
+        if isinstance(color_data, dict):
+            from .models import Color
+            red = color_data.get('red')
+            green = color_data.get('green')
+            blue = color_data.get('blue')
+            alpha = color_data.get('alpha', 255)
+            if all(isinstance(v, int) for v in [red, green, blue]):
+                color = Color(red, green, blue, alpha)
+
+        # Parse status if present
+        status = None
+        status_data = obj_data.get('status')
+        if isinstance(status_data, dict):
+            from .models import TextStatus, FontRecommendation, FontType
+
+            # Parse font recommendation
+            font_rec_data = status_data.get('fontRecommendation')
+            font_rec = None
+            if isinstance(font_rec_data, dict):
+                font_rec = FontRecommendation(
+                    font_name=font_rec_data.get('fontName', ''),
+                    font_type=FontType(font_rec_data.get('fontType', 'SYSTEM')),
+                    similarity_score=font_rec_data.get('similarityScore', 0.0)
+                )
+
+            status = TextStatus(
+                modified=status_data.get('modified', False),
+                encodable=status_data.get('encodable', True),
+                font_type=FontType(status_data.get('fontType', 'UNKNOWN')),
+                font_recommendation=font_rec
+            )
+
+        text_object = TextObjectRef(
+            internal_id=internal_id,
+            position=position,
+            object_type=object_type,
+            text=obj_data.get('text') if isinstance(obj_data.get('text'), str) else None,
+            font_name=obj_data.get('fontName') if isinstance(obj_data.get('fontName'), str) else None,
+            font_size=obj_data.get('fontSize') if isinstance(obj_data.get('fontSize'), (int, float)) else None,
+            line_spacings=line_spacings,
+            color=color,
+            status=status
+        )
+
+        if isinstance(obj_data.get('children'), list) and len(obj_data['children']) > 0:
+            text_object.children = [
+                self._parse_text_object_ref(child_data, f"{internal_id or 'child'}-{index}")
+                for index, child_data in enumerate(obj_data['children'])
+            ]
+
+        return text_object
+
+    def _parse_page_ref(self, obj_data: dict) -> PageRef:
+        """Parse JSON object data into PageRef instance with page-specific properties."""
+        position_data = obj_data.get('position', {})
+        position = self._parse_position(position_data) if position_data else None
+
+        object_type = ObjectType(obj_data['type'])
+
+        # Parse page size if present
+        page_size = None
+        if 'pageSize' in obj_data and isinstance(obj_data['pageSize'], dict):
+            page_size_data = obj_data['pageSize']
+            try:
+                page_size = PageSize.from_dict(page_size_data)
+            except ValueError:
+                page_size = None
+
+        # Parse orientation if present
+        orientation_value = obj_data.get('orientation')
+        orientation = None
+        if isinstance(orientation_value, str):
+            normalized = orientation_value.strip().upper()
+            try:
+                orientation = Orientation(normalized)
+            except ValueError:
+                orientation = None
+        elif isinstance(orientation_value, Orientation):
+            orientation = orientation_value
+
+        return PageRef(
+            internal_id=obj_data.get('internalId'),
+            position=position,
+            type=object_type,
+            page_size=page_size,
+            orientation=orientation
+        )
+
     # Builder Pattern Support
 
     def _paragraph_builder(self) -> 'ParagraphBuilder':
@@ -892,30 +1268,81 @@ class PDFDancer:
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         """Context manager exit - cleanup if needed."""
-        # Could add session cleanup here if API supports it
+        # TODO Could add session cleanup here if API supports it. Cleanup on the server
         pass
 
-    def _to_path_objects(self, path_refs: List[ObjectRef]) -> List[PathObject]:
-        return [PathObject(self, ref.internal_id, ref.type, ref.position) for ref in path_refs]
+    def _to_path_objects(self, refs: List[ObjectRef]) -> List[PathObject]:
+        return [PathObject(self, ref.internal_id, ref.type, ref.position) for ref in refs]
 
-    def _to_paragraph_objects(self, path_refs: List[ObjectRef]) -> List[ParagraphObject]:
-        return [ParagraphObject(self, ref.internal_id, ref.type, ref.position) for ref in path_refs]
+    def _to_paragraph_objects(self, refs: List[TextObjectRef]) -> List[ParagraphObject]:
+        return [ParagraphObject(self, ref) for ref in refs]
 
-    def _to_textline_objects(self, path_refs: List[ObjectRef]) -> List[TextLineObject]:
-        return [TextLineObject(self, ref.internal_id, ref.type, ref.position) for ref in path_refs]
+    def _to_textline_objects(self, refs: List[TextObjectRef]) -> List[TextLineObject]:
+        return [TextLineObject(self, ref) for ref in refs]
 
-    def _to_image_objects(self, path_refs: List[ObjectRef]) -> List[ImageObject]:
-        return [ImageObject(self, ref.internal_id, ref.type, ref.position) for ref in path_refs]
+    def _to_image_objects(self, refs: List[ObjectRef]) -> List[ImageObject]:
+        return [ImageObject(self, ref.internal_id, ref.type, ref.position) for ref in refs]
 
-    def _to_form_objects(self, path_refs: List[ObjectRef]) -> List[FormObject]:
-        return [FormObject(self, ref.internal_id, ref.type, ref.position) for ref in path_refs]
+    def _to_form_objects(self, refs: List[ObjectRef]) -> List[FormObject]:
+        return [FormObject(self, ref.internal_id, ref.type, ref.position) for ref in refs]
 
-    def _to_form_field_objects(self, path_refs: List[FormFieldRef]) -> List[FormFieldObject]:
+    def _to_form_field_objects(self, refs: List[FormFieldRef]) -> List[FormFieldObject]:
         return [FormFieldObject(self, ref.internal_id, ref.type, ref.position, ref.name, ref.value) for ref in
-                path_refs]
+                refs]
 
-    def _to_page_objects(self, path_refs: List[ObjectRef]) -> List[PageClient]:
-        return [PageClient.from_ref(self, ref) for ref in path_refs]
+    def _to_page_objects(self, refs: List[PageRef]) -> List[PageClient]:
+        return [PageClient.from_ref(self, ref) for ref in refs]
 
-    def _to_page_object(self, ref: ObjectRef) -> PageClient:
+    def _to_page_object(self, ref: PageRef) -> PageClient:
         return PageClient.from_ref(self, ref)
+
+    def _to_mixed_objects(self, refs: List[ObjectRef]) -> List:
+        """
+        Convert a list of ObjectRefs to their appropriate object types.
+        Handles mixed object types by checking the type of each ref.
+        """
+        result = []
+        for ref in refs:
+            if ref.type == ObjectType.PARAGRAPH:
+                # Need to convert to TextObjectRef first
+                if isinstance(ref, TextObjectRef):
+                    result.append(ParagraphObject(self, ref))
+                else:
+                    # Re-fetch with proper type
+                    text_refs = self._find_paragraphs(ref.position)
+                    result.extend(self._to_paragraph_objects(text_refs))
+            elif ref.type == ObjectType.TEXT_LINE:
+                if isinstance(ref, TextObjectRef):
+                    result.append(TextLineObject(self, ref))
+                else:
+                    text_refs = self._find_text_lines(ref.position)
+                    result.extend(self._to_textline_objects(text_refs))
+            elif ref.type == ObjectType.IMAGE:
+                result.append(ImageObject(self, ref.internal_id, ref.type, ref.position))
+            elif ref.type == ObjectType.PATH:
+                result.append(PathObject(self, ref.internal_id, ref.type, ref.position))
+            elif ref.type == ObjectType.FORM_X_OBJECT:
+                result.append(FormObject(self, ref.internal_id, ref.type, ref.position))
+            elif ref.type == ObjectType.FORM_FIELD:
+                if isinstance(ref, FormFieldRef):
+                    result.append(FormFieldObject(self, ref.internal_id, ref.type, ref.position, ref.name, ref.value))
+                else:
+                    form_refs = self._find_form_fields(ref.position)
+                    result.extend(self._to_form_field_objects(form_refs))
+        return result
+
+    def select_elements(self):
+        """
+        Select all elements (paragraphs, images, paths, forms) in the document.
+
+        Returns:
+            List of all PDF objects in the document
+        """
+        result = []
+        result.extend(self.select_paragraphs())
+        result.extend(self.select_text_lines())
+        result.extend(self.select_images())
+        result.extend(self.select_paths())
+        result.extend(self.select_forms())
+        result.extend(self.select_form_fields())
+        return result
